#!/bin/bash -p
#
# (c) 2010, Amfeltec Corp
# support@amfeltec.com
#


ROOT=
TMPFILE=/tmp/tmp.$$
INITD=/etc/init.d
MODULES_FILE=/etc/dahdi/modules

system=redhat  # assume redhat
if [ -f /etc/debian_version ]; then
    system=debian
fi

if [ -d /etc/rc.d/init.d ]; then
	INITD=/etc/rc.d/init.d
elif [ -d  /etc/init.d ]; then
	INITD=/etc/init.d
else
	echo "Error: No /etc/init.d directory found!"
	exit 1
fi

err=1;
SNAME="dahdi"
CNAME="Dahdi"
VERSION="DUMMY"

# As for dahdi_dummy, DAHDI should not longer require dahdi_dummy on any supported kernel (Shaun Ruffel, 2012)
# So just add a couple of strings to dahdi/modules:

if [ -f $MODULES_FILE ]; then
	if grep -Fxq "voicesync" "$MODULES_FILE"
	then
		echo	
		echo "--------------------------------------"
		echo "VoiceSync Boot is already configured!"
		echo "Please check file /etc/dahdi/modules."
		echo
		exit 1
	else
		echo  >> $MODULES_FILE
		echo "# Amfeltec Voice Timer" >> $MODULES_FILE
		echo "voicesync" >> $MODULES_FILE
		VERSION="MODULES"
		err=0;
	fi
fi


#Check for Zaptel Startup Script
#In case zaptel startup script exist we will
#attach to zaptel

if [ $VERSION == "DUMMY" ]; then

	if [ -f $INITD/zaptel ]; then
		if grep -Fxq "ztdummy" "$INITD/zaptel"
		then
			cat $INITD/zaptel | sed -e s/ztdummy/voicesync/ > $TMPFILE
			cat $TMPFILE > $INITD/zaptel
			rm $TMPFILE
			SNAME="zaptel"
			CNAME="Zaptel"
			err=0;
		fi
	fi

	if [ -f $INITD/dahdi ]; then
		if grep -Fxq "dahdi_dummy" "$INITD/dahdi"
		then
			cat $INITD/dahdi | sed -e s/dahdi_dummy/voicesync/ > $TMPFILE
			cat $TMPFILE > $INITD/dahdi
			rm $TMPFILE
			SNAME="dahdi"
			CNAME="Dahdi"
			err=0;
		fi
	fi

fi

if [ $err -eq 0 ]; then
	echo
	echo "VoiceSync Boot Setup Complete!"
	echo "--------------------------------------"
	echo

	if [ $VERSION == "MODULES" ]; then
		echo "The voicesync module was added to /etc/dahdi/modules list."
	fi

	if [ $VERSION == "DUMMY" ]; then
		echo "$CNAME $INITD/$SNAME startup script has been updated with VoiceiSync!"
		echo "The $SNAME dummy has now been replaced with voiceisync as the"
		echo "default timing source for $SNAME."
	fi

	echo
	echo "Simply start $SNAME using: $INITD/$SNAME start or service dahdi start"
	echo "No other configuration is necessary!"
	echo
	err=0
else
	echo
	echo
	echo "Error: No Dahdi/Zaptel startup script found in $INITD"
	echo
	echo "Please install and configure Dahdi/Zaptel before VoiceSync"
	echo 
	echo "Example:"
	echo "------- "
	echo "-> cd /usr/src/dahdi-linux-<version>/"
	echo "-> make		#build dahdi"
	echo "-> make install	#install dahdi modules"
	echo "-> make config	#install dahdi boot scripts"
	echo 
	echo "-> cd /usr/src/zaptel-<version>/"
	echo "-> ./configure  (from zaptel 1.4 and higher)"
	echo "-> make		#build zaptel"
	echo "-> make install	#install zaptel modules"
	echo "-> make config	#install zaptel boot scripts"
	echo
	echo "-> cd /usr/src/voicesync-<version>/"
	echo "-> make boot"
	echo
	echo
	err=1
fi

exit $err

